#ifndef _MODULE_DES3_H_
#define _MODULE_DES3_H_
/*
 *             des3 - NBS Data Encryption Standard Library
 *
 *      Copyright (c) 1992 - 96 by CryptoSoft GmbH. All Rights Reserved.
 *                    FON/FAX: +49 (0) 30 7468 2840
 *                      http://www.cryptosoft.com
 *                         des3@cryptosoft.com
 *
 *    Synopsis:	des_init(k8, ks)
 * Description: intializes all arrays and permutation tables for
 *              single DES
 *       Input: k8 -  8 byte key
 *              ks -  key schedule
 *      Output: 0 if OK; >0 if a (semi) weak was selected
 *
 *    Synopsis: des_ecbXcode(in64, out64, ks)
 * Description: encrypts (X=en) or decrypts (X=de) 64-bit inblock to
 *              64-bit outblock using single DES in ECB mode
 *     	 Input: in64  - pointer to 64-bit buffer of input data
 *              out64 - pointer to 64-bit buffer for output data
 *              ks    - key schedule
 *      Output: 0 if OK
 *
 *    Synopsis: des_cbcXcode(in64, out64, iv8, ks)
 * Description: encrypts (X=en) or decrypts (X=de) 64-bit inblock to
 *              64-bit outblock using single DES in CBC mode
 *     	 Input: in64  - pointer to 64-bit buffer of input data
 *              out64 - pointer to 64-bit buffer for output data
 *              iv8   - initalization vector
 *              ks    - key schedule
 *      Output: 0 if OK
 *
 *    Synopsis:	des_3EDEinit(k24, ks1, ks2)
 * Description: intializes all arrays and permutation tables for
 *              triple DES in EDE (encrypt-decrypt-encrypt) mode
 *       Input: k24 -  24 byte key
 *              ks1 -  key schedule
 *              ks2 -  key schedule
 *      Output: 0 if OK; >0 if a (semi) weak was selected
 *
 *    Synopsis: des_ecb3EDEXcode(in64, out64, ks1, ks2)
 * Description: encrypts (X=en) or decrypts (X=de) 64-bit inblock to
 *              64-bit outblock using triple DES in ECB-EDE mode
 *     	 Input: in64  - pointer to 64-bit buffer of input data
 *              out64 - pointer to 64-bit buffer for output data
 *              ks1   - key schedule
 *              ks2   - key schedule
 *      Output: 0 if OK
 *
 *    Synopsis: des_cbc3EDEXcode(in64, out64, iv8, ks1, ks2)
 * Description: encrypts (X=en) or decrypts (X=de) 64-bit inblock to
 *              64-bit outblock using triple DES in CBC-EDE mode
 *     	 Input: in64  - pointer to 64-bit buffer of input data
 *              out64 - pointer to 64-bit buffer for output data
 *              iv8   - initalization vector
 *              ks1   - key schedule
 *              ks2   - key schedule
 *      Output: 0 if OK
 *
 *    Synopsis:	des_3EEEinit(k24, ks1, ks2, ks3)
 * Description: intializes all arrays and permutation tables for
 *              triple DES in EEE (encrypt-encrypt-encrypt) mode
 *       Input: k24 -  24 byte key
 *              ks1 -  key schedule
 *              ks2 -  key schedule
 *              ks3 -  key schedule
 *      Output: 0 if OK; >0 if a (semi) weak was selected
 *
 *    Synopsis: des_ecb3EEEXcode(in64, out64, ks1, ks2, ks3)
 * Description: encrypts (X=en) or decrypts (X=de) 64-bit inblock to
 *              64-bit outblock using triple DES in ECB-EEE mode
 *     	 Input: in64  - pointer to 64-bit buffer of input data
 *              out64 - pointer to 64-bit buffer for output data
 *              ks1   - key schedule
 *              ks2   - key schedule
 *              ks3   - key schedule
 *      Output: 0 if OK
 *
 *    Synopsis: des_cbc3EEEXcode(in64, out64, iv8, ks1, ks2, ks3)
 * Description: encrypts (X=en) or decrypts (X=de) 64-bit inblock to
 *              64-bit outblock using triple DES in CBC-EEE mode
 *     	 Input: in64  - pointer to 64-bit buffer of input data
 *              out64 - pointer to 64-bit buffer for output data
 *              iv8   - initalization vector
 *              ks1   - key schedule
 *              ks2   - key schedule
 *              ks3   - key schedule
 *      Output: 0 if OK
 *
 */


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef unsigned long DES_KS[16][2];

#if defined(WIN16)
#define DECL int far pascal
#define UCHAR unsigned char far
#define DES_PKS DES_KS far *
#endif /* WIN16 */

#if defined(WIN32) || defined(OS2)
#define DECL int pascal
#define UCHAR unsigned char
#define DES_PKS DES_KS *
#endif /* WIN32 */

#if !defined(WIN16) && !defined(WIN32) && !defined(OS2)
#define DECL int
#define UCHAR unsigned char
#define DES_PKS DES_KS *
#endif /* !WIN16 && !WIN32 && !OS2 */

DECL des_init(UCHAR *k8, DES_KS k);
DECL des_ecbencode(UCHAR *i8, UCHAR *o8, DES_KS k);
DECL des_ecbdecode(UCHAR *i8, UCHAR *o8, DES_KS k);
DECL des_cbcencode(UCHAR *i8, UCHAR *o8, UCHAR *iv8, DES_KS k);
DECL des_cbcdecode(UCHAR *i8, UCHAR *o8, UCHAR *iv8, DES_KS k);
DECL des_3EDEinit(UCHAR *k24, DES_KS k1, DES_KS k2);
DECL des_ecb3EDEencode(UCHAR *i8, UCHAR *o8, DES_KS k1, DES_KS k2);
DECL des_ecb3EDEdecode(UCHAR *i8, UCHAR *o8, DES_KS k1, DES_KS k2);
DECL des_cbc3EDEencode(UCHAR *i8, UCHAR *o8, UCHAR *iv8, DES_KS k1, DES_KS k2);
DECL des_cbc3EDEdecode(UCHAR *i8, UCHAR *o8, UCHAR *iv8, DES_KS k1, DES_KS k2);
DECL des_3EEEinit(UCHAR *k24, DES_KS k1, DES_KS k2, DES_KS k3);
DECL des_ecb3EEEencode(UCHAR *i8, UCHAR *o8, DES_KS k1, DES_KS k2, DES_KS k3);
DECL des_ecb3EEEdecode(UCHAR *i8, UCHAR *o8, DES_KS k1, DES_KS k2, DES_KS k3);
DECL des_cbc3EEEencode(UCHAR *i8, UCHAR *o8, UCHAR *iv8, DES_KS k1, DES_KS k2, DES_KS k3);
DECL des_cbc3EEEdecode(UCHAR *i8, UCHAR *o8, UCHAR *iv8, DES_KS k1, DES_KS k2, DES_KS k3);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* _MODULE_DES3_H_ */
