/*
/--------------------------------------------------------------------
|
|      PICLOOK.CPP         application class implementation
|
|      PaintLib MDI MFC graphic files viewer 
|
|      Copyright (c) 1998 Bernard Delme courtesy of Dilys bvba.
|
\--------------------------------------------------------------------
*/

/*
 *	This program hacked from the 'DIBLOOK' MFC example, with
 *	the added capability of multi-format viewing besides BMP.
 *	This was a testbed for paintlib under Win32, really,
 *	and we also use the quick DrawDib bitmap engine.
 *	Reuse at will in derived work; just acknowledge their due credit
 *	to the authors of paintlib and the decoding libraries it relies on.
 */

#include "stdafx.h"
#include "piclook.h"
#include "docman.h"
#include "mainfrm.h"
#include "dibvfw.h"
#include "doc.h"
#include "view.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

// here for easy modification, Ulrich!
const char paintlib_url[] = "http://www.cs.tu-berlin.de/~uzadow/paintlib";

/////////////////////////////////////////////////////////////////////////////
// CPaintLibViewerApp

BEGIN_MESSAGE_MAP(CPaintLibViewerApp, CWinApp)
	//{{AFX_MSG_MAP(CPaintLibViewerApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPaintLibViewerApp construction
// Place all significant initialization in InitInstance

CPaintLibViewerApp::CPaintLibViewerApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CPaintLibViewerApp object

CPaintLibViewerApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CPaintLibViewerApp initialization

BOOL CPaintLibViewerApp::InitInstance()
{
	// Standard initialization
	SetRegistryKey( "PaintLib" );
	LoadStdProfileSettings(5);  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CMultiDocTemplate* pDocTemplate;
	// BMP aware, and more ...
	pDocTemplate = new CMultiDocTemplate(
		IDR_BMPTYPE,
		RUNTIME_CLASS(CPLViewerDoc),
		RUNTIME_CLASS(CMDIChildWnd),		// standard MDI child frame
		//RUNTIME_CLASS(CChildFrame),		// derived MDI child frame
		RUNTIME_CLASS(CPLView));
	ASSERT(m_pDocManager == NULL);
	m_pDocManager = new CDocManagerEx;
	m_pDocManager->AddDocTemplate(pDocTemplate);

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();
	m_pMainWnd = pMainFrame;

	// enable file manager drag/drop and DDE Execute open
	m_pMainWnd->DragAcceptFiles();

	// Uncomment this to have PicLook register itself as default viewer
	// for image files.
	// EnableShellOpen();
	// RegisterShellFileTypes(TRUE);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// MFC faq says this avoids 1st (empty) doc
	if (cmdInfo.m_nShellCommand == CCommandLineInfo::FileNew)
		cmdInfo.m_nShellCommand = CCommandLineInfo::FileNothing;

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnCredits();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
	BOOL m_bCreditsShown;
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
    m_bCreditsShown = TRUE;
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	ON_BN_CLICKED(ID_CREDITS, OnCredits)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CPaintLibViewerApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// paintlib's url gets injected in a read-only edit field
	// so one can select it for pasting
	// maybe we could launch the default browser using ShellExecute...
	GetDlgItem( IDC_URL )->SetWindowText( paintlib_url );
	//((CEdit *) GetDlgItem( IDC_URL ))->SetSel( 0, -1 );

	CenterWindow();
	// hide the credits at startup
	OnCredits();
	
	return TRUE;
}

void CAboutDlg::OnCredits() 
{
	m_bCreditsShown = ! m_bCreditsShown;
    	
	RECT dlg_rect;
	GetWindowRect( &dlg_rect );
	// these size adjustments are scaled to 'dialog units'
	// to (hopefully) work at all resolutions...
	if (m_bCreditsShown)
		dlg_rect.bottom += 7 * HIWORD(::GetDialogBaseUnits());
	else
	    dlg_rect.bottom -= 7 * HIWORD(::GetDialogBaseUnits());

	MoveWindow( &dlg_rect, TRUE );
}
